package cls.logic;

import cls.domain.ApplicationTable;
import cls.domain.BallLightingTable;
import cls.domain.BallMoveEffectTable;
import cls.domain.GameCompositionTable;
import cls.domain.GameTable;
import cls.domain.PlayerTable;
import cls.domain.TimeZoneTable;
import cls.logic.composition.GameComposition;
import okay.datetime.LocalDate;

/**
 *
 * @author Robert Vokáč e-mail: robertvokac1@gmail.com
 */
public class Player {

    /**
     * This is player Id. If no player is logged in, Id is zero.
     */
    private final int id;
    private final Logic logic;

    /**
     *
     * @param playerId
     * @param logic
     */
    public Player(int playerId, Logic logic) {
        this.id = playerId;
        this.logic = logic;
    }

    public ApplicationLook getApplicationLook() {
        return new ApplicationLookForPlayer(this.id);
    }

    /**
     *
     * @return Player Id used to identify player in database
     */
    public int getPlayerId() {
        return this.id;
    }

    /**
     *
     * @param value
     */
    public void setKeepingSignedIn(boolean value) {
        int tempPlayerId;
        if (value) {
            tempPlayerId = id;
        } else {
            tempPlayerId = 0;
        }
        ApplicationTable.setAutomaticallyLoggedInPlayerId(tempPlayerId);
    }

    /**
     *
     */
    public GameComposition getLastPlayedGameComposition() {
        int lastPlayedGameCompositionId = GameTable.getLastPlayedGameCompositionId(this.id);
        return GameCompositionTable.getGameComposition(lastPlayedGameCompositionId);
    }

    /**
     *
     * @return Nick of logged in player.
     */
    public String getPlayerNick() {
        return PlayerTable.getNick(id);
    }

    /**
     *
     * @param playerNick
     * @param newPassword
     */
    public void setPlayerNick(String playerNick, String password) {
        if (!PlayerTable.areNickAndPasswordOK(this.getPlayerNick(), password)) {
            return;
        }
        PlayerTable.setNick(id, playerNick);
    }

    /**
     *
     * @param oldPassword
     * @param newPassword
     * @param confirmNewPassword
     */
    public void updatePassword(String oldPassword, String newPassword, String confirmNewPassword) {
        if ((PlayerTable.areNickAndPasswordOK(PlayerTable.getNick(id), oldPassword)) && (newPassword.equals(confirmNewPassword))) {
            PlayerTable.updatePassword(id, newPassword);
        } else {
            throw new cls.ColorLinesSequelRuntimeException("Password was not changed.");
        }

    }

    /**
     *
     * @return
     */
    public String getBallLightingName() {
        int ballLightingId = PlayerTable.getBallLightingId(id);
        return BallLightingTable.getName(ballLightingId);
    }

    /**
     *
     * @param name
     */
    public void setBallLightingName(String name) {
        int ballLightingId = BallLightingTable.getId(name);
        PlayerTable.setBallLightingId(id, ballLightingId);
    }

    /**
     *
     * @return
     */
    public boolean getLinesAround() {
        return PlayerTable.getLinesAround(id);
    }

    /**
     *
     * @param value
     */
    public void setLinesAround(boolean value) {
        PlayerTable.setShowLinesAroundBall(id, value ? 1 : 0);
    }

    /**
     *
     * @return
     */
    public boolean getShowWhereABallCanBeMoved() {
        return PlayerTable.getShowWhereABallCanBeMoved(id);
    }

    /**
     *
     * @param value
     */
    public void setShowWhereABallCanBeMoved(boolean value) {
        PlayerTable.setWhereABallCanBeMoved(id, value ? 1 : 0);
    }

    /**
     *
     * @return
     */
    public boolean getHighlightFieldAfterBallExploded() {
        return PlayerTable.getHighlightFieldAfterBallExploded(id);
    }

    /**
     *
     * @param value
     */
    public void setShowHighlightFieldAfterBallExploded(boolean value) {
        PlayerTable.setHighlightFieldsAfterBallExploded(id, value ? 1 : 0);
    }

    /**
     *
     * @return
     */
    public String getBallMoveEffectName() {
        int ballMoveEffectId = PlayerTable.getBallMoveEffectId(id);
        return BallMoveEffectTable.getName(ballMoveEffectId);
    }

    /**
     *
     * @param name
     */
    public void setBallMoveEffectName(String name) {
        int ballMoveEffectId = BallMoveEffectTable.getId(name);
        PlayerTable.setBallMoveEffectId(id, ballMoveEffectId);
    }

    /**
     *
     * @return
     */
    public String getName() {
        return PlayerTable.getName(id);
    }

    /**
     *
     * @param name
     */
    public void setName(String name) {
        PlayerTable.setName(id, name);
    }

    /**
     *
     * @return
     */
    public String getSurname() {
        return PlayerTable.getSurname(id);
    }

    /**
     *
     * @param surname
     */
    public void setSurname(String surname) {
        PlayerTable.setSurname(id, surname);
    }

    /**
     *
     * @return
     */
    public String getSex() {
        return PlayerTable.getSex(id);
    }

    /**
     *
     * @param sexName
     */
    public void setSex(String sex) {
        PlayerTable.setSex(id, sex);
    }

    /**
     *
     * @return
     */
    public LocalDate getDateOfBirth() {
        return PlayerTable.getDateOfBirth(id);
    }

    /**
     *
     * @param localDate
     */
    public void setDateOfBirth(LocalDate localDate) {
        PlayerTable.setDateOfBirth(id, localDate);
    }

    /**
     *
     * @return
     */
    public String getTimeZoneName() {
        int timeZoneId = PlayerTable.getTimeZoneId(id);
        return TimeZoneTable.getName(timeZoneId);
    }

    /**
     *
     * @param timeZoneName
     */
    public void setTimeZoneName(String timeZoneName) {
        int timeZoneId = TimeZoneTable.getId(timeZoneName);
        PlayerTable.updateTimeZoneId(id, timeZoneId);
    }

}
